"use strict";

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { useEffect, useMemo, useState } from 'react';
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native';
import { jsx as _jsx } from "react/jsx-runtime";
const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace'
});
const engine = createJavaScriptRegexEngine({
  forgiving: true,
  simulation: true,
  cache: new Map()
});
const SyntaxHighlighter = ({
  text,
  language,
  languageId,
  theme,
  fontSize = 14,
  scrollViewProps,
  textStyle,
  containerStyle
}) => {
  const stylesheet = useMemo(() => {
    const processedStyles = new Map();
    if (!theme.tokenColors) {
      console.warn('Theme does not contain tokenColors');
      return processedStyles;
    }
    theme.tokenColors.forEach(({
      scope,
      settings
    }) => {
      if (!scope || !settings) return;
      const scopes = Array.isArray(scope) ? scope : [scope];
      scopes.forEach(singleScope => {
        processedStyles.set(singleScope, {
          color: settings.foreground,
          fontStyle: settings.fontStyle
        });
      });
    });
    return processedStyles;
  }, [theme]);
  const [tokens, setTokens] = useState([]);
  const [error, setError] = useState(null);
  useEffect(() => {
    let mounted = true;
    async function fetchTokens() {
      if (!text) {
        setTokens([]);
        return;
      }
      try {
        const highlighter = await createHighlighterCore({
          langs: language ? [language] : [],
          themes: [theme],
          engine
        });
        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme,
          includeExplanation: true
        });
        if (mounted) {
          setTokens(result.tokens || []);
          setError(null);
        }
      } catch (err) {
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'));
          setTokens([[{
            content: text,
            color: '#d8dee9',
            offset: 0
          }]]);
        }
      }
    }
    fetchTokens();
    return () => {
      mounted = false;
    };
  }, [text, language, languageId, theme]);
  const getStylesForToken = token => {
    const firstScope = token.explanation?.[0]?.scopes[0];
    const style = firstScope ? stylesheet.get(firstScope.scopeName) || {} : {};
    if (style.fontStyle && style.fontStyle !== 'italic' && style.fontStyle !== 'normal') {
      style.fontStyle = undefined;
    }
    return [{
      ...style,
      fontStyle: style.fontStyle === 'italic' || style.fontStyle === 'normal' ? style.fontStyle : undefined
    }];
  };
  const renderTokens = tokens => {
    return tokens.map((token, index) => {
      const styles = StyleSheet.flatten([textStyle, getStylesForToken(token)]);
      return /*#__PURE__*/_jsx(Text, {
        style: [{
          fontFamily: FONT_FAMILY,
          fontSize,
          color: styles?.color || '#000',
          // Default color if undefined
          fontStyle: styles?.fontStyle === 'italic' ? 'italic' : 'normal',
          fontWeight: styles?.fontStyle?.includes('bold') ? 'bold' : 'normal'
        }, textStyle || {} // Ensure textStyle is always an object
        ],
        children: token.content
      }, index);
    });
  };
  const backgroundColor = theme.colors?.['editor.background'] || '#ffffff';
  if (error) {
    return /*#__PURE__*/_jsx(ScrollView, {
      horizontal: true,
      ...scrollViewProps,
      style: [{
        flex: 1,
        minHeight: 20,
        backgroundColor
      }, scrollViewProps?.style],
      contentContainerStyle: [{
        padding: 16
      }, containerStyle],
      children: /*#__PURE__*/_jsx(Text, {
        style: [{
          fontFamily: FONT_FAMILY,
          fontSize
        }, textStyle],
        children: text
      })
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    horizontal: true,
    ...scrollViewProps,
    style: [{
      flex: 1,
      minHeight: 20,
      backgroundColor
    }, scrollViewProps?.style],
    contentContainerStyle: [{
      padding: 16
    }, containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((line, index) => /*#__PURE__*/_jsx(View, {
        style: {
          flexDirection: 'row',
          flexWrap: 'wrap'
        },
        children: renderTokens(line)
      }, index))
    })
  });
};
export default SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map