import type { LanguageInput, ThemedToken, ThemeRegistrationAny, TokensResult } from '@shikijs/types'
import type { FunctionComponent, ReactNode } from 'react'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'

const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface SyntaxHighlighterProps {
  text: string
  language?: LanguageInput
  languageId?: string
  theme: ThemeRegistrationAny
  fontSize?: number
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const engine = createJavaScriptRegexEngine({ forgiving: true, simulation: true, cache: new Map() })

const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const stylesheet = useMemo(() => {
    const processedStyles = new Map<string, { color?: string, fontStyle?: string }>()

    if (!theme.tokenColors) {
      console.warn('Theme does not contain tokenColors')
      return processedStyles
    }

    theme.tokenColors.forEach(({ scope, settings }) => {
      if (!scope || !settings)
        return
      const scopes = Array.isArray(scope) ? scope : [scope]
      scopes.forEach((singleScope) => {
        processedStyles.set(singleScope, {
          color: settings.foreground,
          fontStyle: settings.fontStyle,
        })
      })
    })

    return processedStyles
  }, [theme])

  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    let mounted = true

    async function fetchTokens() {
      if (!text) {
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighterCore({
          langs: language ? [language] : [],
          themes: [theme],
          engine,
        })
        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme,
          includeExplanation: true,
        }) as TokensResult

        if (mounted) {
          setTokens(result.tokens || [])
          setError(null)
        }
      }
      catch (err) {
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'))
          setTokens([[{ content: text, color: '#d8dee9', offset: 0 }]])
        }
      }
    }

    fetchTokens()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  const getStylesForToken = (token: ThemedToken): TextStyle[] => {
    const firstScope = token.explanation?.[0]?.scopes[0]
    const style = firstScope ? stylesheet.get(firstScope.scopeName) || {} : {}
    if (style.fontStyle && style.fontStyle !== 'italic' && style.fontStyle !== 'normal') {
      style.fontStyle = undefined
    }
    return [
      {
        ...style,
        fontStyle: style.fontStyle === 'italic' || style.fontStyle === 'normal' ? style.fontStyle : undefined,
      },
    ]
  }

  const renderTokens = (tokens: ThemedToken[]): ReactNode => {
    return tokens.map((token, index) => {
      const styles = StyleSheet.flatten([textStyle, getStylesForToken(token)])

      return (
        <Text
          key={index}
          style={[
            {
              fontFamily: FONT_FAMILY,
              fontSize,
              color: styles?.color || '#000', // Default color if undefined
              fontStyle: styles?.fontStyle === 'italic' ? 'italic' : 'normal',
              fontWeight: styles?.fontStyle?.includes('bold') ? 'bold' : 'normal',
            },
            textStyle || {}, // Ensure textStyle is always an object
          ]}
        >
          {token.content}
        </Text>
      )
    })
  }

  const backgroundColor = theme.colors?.['editor.background'] || '#ffffff'

  if (error) {
    return (
      <ScrollView horizontal {...scrollViewProps} style={[{ flex: 1, minHeight: 20, backgroundColor }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 16 }, containerStyle]}>
        <Text style={[{ fontFamily: FONT_FAMILY, fontSize }, textStyle]}>{text}</Text>
      </ScrollView>
    )
  }

  return (
    <ScrollView horizontal {...scrollViewProps} style={[{ flex: 1, minHeight: 20, backgroundColor }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 16 }, containerStyle]}>
      <View>
        {tokens.map((line, index) => (
          <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
            {renderTokens(line)}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
